@@ -182,7 +182,7 @@ static char *vips_pdfium_errors[] = {
 	"page not found or content error"
 };
 
-static GMutex *vips_pdfium_mutex = NULL;
+static GMutex vips_pdfium_mutex;
 
 static void
 vips_pdfium_error(void)
@@ -199,14 +199,14 @@ vips_pdfium_error(void)
 static void
 vips_foreign_load_pdf_close(VipsForeignLoadPdf *pdf)
 {
-	g_mutex_lock(vips_pdfium_mutex);
+	g_mutex_lock(&vips_pdfium_mutex);
 
 	VIPS_FREEF(FPDF_ClosePage, pdf->page);
 	VIPS_FREEF(FPDFDOC_ExitFormFillEnvironment, pdf->form);
 	VIPS_FREEF(FPDF_CloseDocument, pdf->doc);
 	VIPS_UNREF(pdf->source);
 
-	g_mutex_unlock(vips_pdfium_mutex);
+	g_mutex_unlock(&vips_pdfium_mutex);
 }
 
 static void
@@ -277,7 +277,7 @@ vips_foreign_load_pdf_build(VipsObject *object)
 	VIPS_ONCE(&once, vips_pdfium_init_cb, NULL);
 
 	if (!vips_object_argument_isset(object, "scale"))
-		pdf->scale = pdf->dpi / 72.0;
+		pdf->scale = pdf->dpi / 72.0; // FIXME: Invalidates operation cache
 
 	pdf->form_callbacks.version = 2;
 
@@ -297,11 +297,11 @@ vips_foreign_load_pdf_build(VipsObject *object)
 		pdf->file_access.m_GetBlock = vips_pdfium_GetBlock;
 		pdf->file_access.m_Param = pdf;
 
-		g_mutex_lock(vips_pdfium_mutex);
+		g_mutex_lock(&vips_pdfium_mutex);
 
 		if (!(pdf->doc = FPDF_LoadCustomDocument(&pdf->file_access,
 				  pdf->password))) {
-			g_mutex_unlock(vips_pdfium_mutex);
+			g_mutex_unlock(&vips_pdfium_mutex);
 			vips_pdfium_error();
 			vips_error("pdfload",
 				_("%s: unable to load"),
@@ -312,7 +312,7 @@ vips_foreign_load_pdf_build(VipsObject *object)
 
 		if (!(pdf->form = FPDFDOC_InitFormFillEnvironment(pdf->doc,
 				  &pdf->form_callbacks))) {
-			g_mutex_unlock(vips_pdfium_mutex);
+			g_mutex_unlock(&vips_pdfium_mutex);
 			vips_pdfium_error();
 			vips_error("pdfload",
 				_("%s: unable to initialize form fill environment"),
@@ -321,13 +321,11 @@ vips_foreign_load_pdf_build(VipsObject *object)
 			return -1;
 		}
 
-		g_mutex_unlock(vips_pdfium_mutex);
+		g_mutex_unlock(&vips_pdfium_mutex);
 	}
 
-	if (VIPS_OBJECT_CLASS(vips_foreign_load_pdf_parent_class)->build(object))
-		return -1;
-
-	return 0;
+	return VIPS_OBJECT_CLASS(vips_foreign_load_pdf_parent_class)
+		->build(object);
 }
 
 static VipsForeignFlags
@@ -350,7 +348,7 @@ vips_foreign_load_pdf_get_page(VipsForeignLoadPdf *pdf, int page_no)
 	if (pdf->current_page != page_no) {
 		VipsObjectClass *class = VIPS_OBJECT_GET_CLASS(pdf);
 
-		g_mutex_lock(vips_pdfium_mutex);
+		g_mutex_lock(&vips_pdfium_mutex);
 
 		VIPS_FREEF(FPDF_ClosePage, pdf->page);
 		pdf->current_page = -1;
@@ -360,7 +358,7 @@ vips_foreign_load_pdf_get_page(VipsForeignLoadPdf *pdf, int page_no)
 #endif /*DEBUG*/
 
 		if (!(pdf->page = FPDF_LoadPage(pdf->doc, page_no))) {
-			g_mutex_unlock(vips_pdfium_mutex);
+			g_mutex_unlock(&vips_pdfium_mutex);
 			vips_pdfium_error();
 			vips_error(class->nickname,
 				_("unable to load page %d"), page_no);
@@ -368,7 +366,7 @@ vips_foreign_load_pdf_get_page(VipsForeignLoadPdf *pdf, int page_no)
 		}
 		pdf->current_page = page_no;
 
-		g_mutex_unlock(vips_pdfium_mutex);
+		g_mutex_unlock(&vips_pdfium_mutex);
 	}
 
 	return 0;
@@ -412,7 +410,7 @@ vips_foreign_load_pdf_set_image(VipsForeignLoadPdf *pdf, VipsImage *out)
 	vips_image_set_int(out, "pdf-n_pages", pdf->n_pages);
 	vips_image_set_int(out, VIPS_META_N_PAGES, pdf->n_pages);
 
-	g_mutex_lock(vips_pdfium_mutex);
+	g_mutex_lock(&vips_pdfium_mutex);
 
 	for (i = 0; i < n_metadata; i++) {
 		VipsForeignLoadPdfMetadata *metadata =
@@ -436,7 +434,7 @@ vips_foreign_load_pdf_set_image(VipsForeignLoadPdf *pdf, VipsImage *out)
 		}
 	}
 
-	g_mutex_unlock(vips_pdfium_mutex);
+	g_mutex_unlock(&vips_pdfium_mutex);
 
 	/* We need pixels/mm for vips.
 	 */
@@ -463,14 +461,14 @@ vips_foreign_load_pdf_header(VipsForeignLoad *load)
 	printf("vips_foreign_load_pdf_header: %p\n", pdf);
 #endif /*DEBUG*/
 
-	g_mutex_lock(vips_pdfium_mutex);
+	g_mutex_lock(&vips_pdfium_mutex);
 	pdf->n_pages = FPDF_GetPageCount(pdf->doc);
-	g_mutex_unlock(vips_pdfium_mutex);
+	g_mutex_unlock(&vips_pdfium_mutex);
 
 	/* @n == -1 means until the end of the doc.
 	 */
 	if (pdf->n == -1)
-		pdf->n = pdf->n_pages - pdf->page_no;
+		pdf->n = pdf->n_pages - pdf->page_no; // FIXME: Invalidates operation cache
 
 	if (pdf->page_no + pdf->n > pdf->n_pages ||
 		pdf->page_no < 0 ||
@@ -498,9 +496,9 @@ vips_foreign_load_pdf_header(VipsForeignLoad *load)
 		 * does round to nearest. Without this, things like
 		 * shrink-on-load will break.
 		 */
-		pdf->pages[i].width = VIPS_RINT(
+		pdf->pages[i].width = rint(
 			FPDF_GetPageWidth(pdf->page) * pdf->scale);
-		pdf->pages[i].height = VIPS_RINT(
+		pdf->pages[i].height = rint(
 			FPDF_GetPageHeight(pdf->page) * pdf->scale);
 
 		/* PDFium allows page width or height to be less than 1 (!!).
@@ -564,7 +562,6 @@ vips_foreign_load_pdf_generate(VipsRegion *out_region,
 
 	int top;
 	int i;
-	int y;
 
 	/*
 	printf("vips_foreign_load_pdf_generate: "
@@ -587,58 +584,58 @@ vips_foreign_load_pdf_generate(VipsRegion *out_region,
 
 	top = r->top;
 	while (top < VIPS_RECT_BOTTOM(r)) {
-		VipsRect rect;
 		FPDF_BITMAP bitmap;
 
+		/* Is the rect within this page? It might not be if the output is more
+		 * than one tile wide and this page is narrower.
+		 */
+		VipsRect rect;
 		vips_rect_intersectrect(r, &pdf->pages[i], &rect);
+		if (rect.width > 0 &&
+			rect.height > 0) {
 
-		if (vips_foreign_load_pdf_get_page(pdf, pdf->page_no + i))
-			return -1;
+			if (vips_foreign_load_pdf_get_page(pdf, pdf->page_no + i))
+				return -1;
 
-		vips__worker_lock(vips_pdfium_mutex);
+			vips__worker_lock(&vips_pdfium_mutex);
 
-		/* 4 means RGBA.
-		 */
-		bitmap = FPDFBitmap_CreateEx(rect.width, rect.height, 4,
-			VIPS_REGION_ADDR(out_region, rect.left, rect.top),
-			VIPS_REGION_LSKIP(out_region));
+			/* 4 means RGBA.
+			 */
+			bitmap = FPDFBitmap_CreateEx(rect.width, rect.height, 4,
+				VIPS_REGION_ADDR(out_region, rect.left, rect.top),
+				VIPS_REGION_LSKIP(out_region));
 
-		/* Only paint the background if there's no transparency.
-		 */
-		if (!FPDFPage_HasTransparency(pdf->page)) {
-			FPDF_DWORD ink = *((guint32 *) pdf->ink);
+			/* Only paint the background if there's no transparency.
+			 */
+			if (!FPDFPage_HasTransparency(pdf->page)) {
+				FPDF_DWORD ink = *((guint32 *) pdf->ink);
 
-			FPDFBitmap_FillRect(bitmap,
-				0, 0, rect.width, rect.height, ink);
+				FPDFBitmap_FillRect(bitmap,
+					0, 0, rect.width, rect.height, ink);
+			}
+
+			// pdfium writes bgra by default, we need rgba
+			FPDF_RenderPageBitmap(bitmap, pdf->page,
+				pdf->pages[i].left - rect.left,
+				pdf->pages[i].top - rect.top,
+				pdf->pages[i].width, pdf->pages[i].height,
+				0, FPDF_ANNOT | FPDF_REVERSE_BYTE_ORDER);
+
+			FPDF_FFLDraw(pdf->form, bitmap, pdf->page,
+				pdf->pages[i].left - rect.left,
+				pdf->pages[i].top - rect.top,
+				pdf->pages[i].width, pdf->pages[i].height,
+				0, FPDF_ANNOT | FPDF_REVERSE_BYTE_ORDER);
+
+			FPDFBitmap_Destroy(bitmap);
+
+			g_mutex_unlock(&vips_pdfium_mutex);
 		}
 
-		FPDF_RenderPageBitmap(bitmap, pdf->page,
-			pdf->pages[i].left - rect.left,
-			pdf->pages[i].top - rect.top,
-			pdf->pages[i].width, pdf->pages[i].height,
-			0, 0);
-
-		FPDF_FFLDraw(pdf->form, bitmap, pdf->page,
-			pdf->pages[i].left - rect.left,
-			pdf->pages[i].top - rect.top,
-			pdf->pages[i].width, pdf->pages[i].height,
-			0, 0);
-
-		FPDFBitmap_Destroy(bitmap);
-
-		g_mutex_unlock(vips_pdfium_mutex);
-
 		top += rect.height;
 		i += 1;
 	}
 
-	/* PDFium writes BGRA, we must swap.
-	 */
-	for (y = 0; y < r->height; y++)
-		vips__bgra2rgba(
-			(guint32 *) VIPS_REGION_ADDR(out_region, r->left, r->top + y),
-			r->width);
-
 	return 0;
 }
 
@@ -677,28 +674,13 @@ vips_foreign_load_pdf_load(VipsForeignLoad *load)
 	return 0;
 }
 
-static void *
-vips_foreign_load_pdf_once_init(void *client)
-{
-	/* We must make the mutex on class init (not _build) since we
-	 * can lock even if build is not called.
-	 */
-	vips_pdfium_mutex = vips_g_mutex_new();
-
-	return NULL;
-}
-
 static void
 vips_foreign_load_pdf_class_init(VipsForeignLoadPdfClass *class)
 {
-	static GOnce once = G_ONCE_INIT;
-
 	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
 	VipsObjectClass *object_class = (VipsObjectClass *) class;
 	VipsForeignLoadClass *load_class = (VipsForeignLoadClass *) class;
 
-	VIPS_ONCE(&once, vips_foreign_load_pdf_once_init, NULL);
-
 	gobject_class->dispose = vips_foreign_load_pdf_dispose;
 	gobject_class->set_property = vips_object_set_property;
 	gobject_class->get_property = vips_object_get_property;
@@ -791,11 +773,6 @@ vips_foreign_load_pdf_file_header(VipsForeignLoad *load)
 		->header(load);
 }
 
-static const char *vips_foreign_pdf_suffs[] = {
-	".pdf",
-	NULL
-};
-
 static int
 vips_foreign_load_pdf_file_build(VipsObject *object)
 {
@@ -814,33 +791,6 @@ vips_foreign_load_pdf_file_build(VipsObject *object)
 		->build(object);
 }
 
-static gboolean
-vips_foreign_load_pdf_is_a_buffer(const void *buf, size_t len)
-{
-	const guchar *str = (const guchar *) buf;
-
-	if (len >= 4 &&
-		str[0] == '%' &&
-		str[1] == 'P' &&
-		str[2] == 'D' &&
-		str[3] == 'F')
-		return 1;
-
-	return 0;
-}
-
-static gboolean
-vips_foreign_load_pdf_is_a(const char *filename)
-{
-	unsigned char buf[4];
-
-	if (vips__get_bytes(filename, buf, 4) == 4 &&
-		vips_foreign_load_pdf_is_a_buffer(buf, 4))
-		return 1;
-
-	return 0;
-}
-
 static void
 vips_foreign_load_pdf_file_class_init(
 	VipsForeignLoadPdfFileClass *class)
@@ -857,9 +807,9 @@ vips_foreign_load_pdf_file_class_init(
 	object_class->description = _("load PDF from file");
 	object_class->build = vips_foreign_load_pdf_file_build;
 
-	foreign_class->suffs = vips_foreign_pdf_suffs;
+	foreign_class->suffs = vips__pdf_suffs;
 
-	load_class->is_a = vips_foreign_load_pdf_is_a;
+	load_class->is_a = vips__pdf_is_a_file;
 	load_class->header = vips_foreign_load_pdf_file_header;
 
 	VIPS_ARG_STRING(class, "filename", 1,
@@ -920,7 +870,7 @@ vips_foreign_load_pdf_buffer_class_init(
 	object_class->description = _("load PDF from buffer");
 	object_class->build = vips_foreign_load_pdf_buffer_build;
 
-	load_class->is_a_buffer = vips_foreign_load_pdf_is_a_buffer;
+	load_class->is_a_buffer = vips__pdf_is_a_buffer;
 
 	VIPS_ARG_BOXED(class, "buffer", 1,
 		_("Buffer"),
@@ -962,18 +912,6 @@ vips_foreign_load_pdf_source_build(VipsObject *object)
 		->build(object);
 }
 
-static gboolean
-vips_foreign_load_pdf_source_is_a_source(VipsSource *source)
-{
-	const unsigned char *p;
-
-	return (p = vips_source_sniff(source, 4)) &&
-		p[0] == '%' &&
-		p[1] == 'P' &&
-		p[2] == 'D' &&
-		p[3] == 'F';
-}
-
 static void
 vips_foreign_load_pdf_source_class_init(
 	VipsForeignLoadPdfSourceClass *class)
@@ -992,7 +930,7 @@ vips_foreign_load_pdf_source_class_init(
 
 	operation_class->flags |= VIPS_OPERATION_NOCACHE;
 
-	load_class->is_a_source = vips_foreign_load_pdf_source_is_a_source;
+	load_class->is_a_source = vips__pdf_is_a_source;
 
 	VIPS_ARG_OBJECT(class, "source", 1,
 		_("Source"),
