@@ -1,4 +1,4 @@
-/* error.c --- error message handling
+/* error.c -- error message handling
  *
  * Copyright: N. Dessipris
  * Written on: 18/03/1991
@@ -62,7 +62,6 @@
 #include <vips/vips.h>
 #include <vips/internal.h>
 #include <vips/buf.h>
-#include <vips/thread.h>
 #include <vips/debug.h>
 
 #ifdef G_OS_WIN32
@@ -70,64 +69,6 @@
 #include <lmerr.h>
 #endif /*G_OS_WIN32*/
 
-/**
- * SECTION: errors
- * @short_description: error messages and error handling
- * @stability: Stable
- * @include: vips/vips.h
- *
- * VIPS maintains an error buffer (a log of localised text messages),
- * a set of functions
- * for adding messages, and a way to access and clear the buffer.
- *
- * The error buffer is global, that is, it is shared between all threads. You
- * can add to the buffer from any thread (there is a lock to prevent
- * corruption), but it's sensible to only read and clear the buffer from the
- * main thread of execution.
- *
- * The general principle is: if you detect an error, log a message for the
- * user. If a function you call detects an error, just propagate it and don't
- * add another message.
- *
- * |[
- * VipsImage *im;
- *
- * if (!(im = vips_image_new_from_file(filename, NULL)))
- *     // vips_image_new_from_file() will set a message, we don't need to
- *     return -1;
- *
- * if (vips_image_get_width(im) &lt; 100) {
- *     // we have detected an error, we must set a message
- *     vips_error("myprogram", "%s", _("width too small"));
- *     return -1;
- * }
- * ]|
- *
- * The domain argument most of these functions take is not localised and is
- * supposed to indicate the component which failed.
- *
- * libvips uses g_warning() and g_info() to send warning and information
- * messages to the user. You can use the usual glib mechanisms to display or
- * divert these messages. For example, info messages are hidden by default, but
- * you can see them with:
- *
- * |[
- * $ G_MESSAGES_DEBUG=VIPS vipsthumbnail k2.jpg
- * VIPS-INFO: thumbnailing k2.jpg
- * VIPS-INFO: selected loader is VipsForeignLoadJpegFile
- * VIPS-INFO: input size is 1450 x 2048
- * VIPS-INFO: loading jpeg with factor 8 pre-shrink
- * VIPS-INFO: converting to processing space srgb
- * VIPS-INFO: residual reducev by 0.5
- * VIPS-INFO: 13 point mask
- * VIPS-INFO: using vector path
- * VIPS-INFO: residual reduceh by 0.5
- * VIPS-INFO: 13 point mask
- * VIPS-INFO: thumbnailing k2.jpg as ./tn_k2.jpg
- * ]|
- *
- */
-
 /* Make global array to keep the error message buffer.
  */
 #define VIPS_MAX_ERROR (10240)
@@ -138,16 +79,16 @@ static int vips_error_freeze_count = 0;
 /**
  * vips_error_freeze:
  *
- * Stop errors being logged. Use vips_error_thaw() to unfreeze. You can
+ * Stop errors being logged. Use [func@error_thaw] to unfreeze. You can
  * nest freeze/thaw pairs.
  */
 void
 vips_error_freeze(void)
 {
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	g_assert(vips_error_freeze_count >= 0);
 	vips_error_freeze_count += 1;
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 }
 
 /**
@@ -158,10 +99,10 @@ vips_error_freeze(void)
 void
 vips_error_thaw(void)
 {
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	vips_error_freeze_count -= 1;
 	g_assert(vips_error_freeze_count >= 0);
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 }
 
 /**
@@ -170,7 +111,8 @@ vips_error_thaw(void)
  * Get a pointer to the start of the error buffer as a C string.
  * The string is owned by the error system and must not be freed.
  *
- * See also: vips_error_clear().
+ * ::: seealso
+ *     [func@error_clear].
  *
  * Returns: the error buffer as a C string which must not be freed
  */
@@ -179,9 +121,9 @@ vips_error_buffer(void)
 {
 	const char *msg;
 
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	msg = vips_buf_all(&vips_error_buf);
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 
 	return msg;
 }
@@ -198,10 +140,10 @@ vips_error_buffer_copy(void)
 {
 	char *msg;
 
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	msg = g_strdup(vips_buf_all(&vips_error_buf));
 	vips_buf_rewind(&vips_error_buf);
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 
 	return msg;
 }
@@ -218,12 +160,13 @@ vips_error_buffer_copy(void)
 /**
  * vips_verror:
  * @domain: the source of the error
- * @fmt: printf()-style format string for the error
+ * @fmt: `printf()`-style format string for the error
  * @ap: arguments to the format string
  *
  * Append a message to the error buffer.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  */
 void
 vips_verror(const char *domain, const char *fmt, va_list ap)
@@ -242,7 +185,7 @@ vips_verror(const char *domain, const char *fmt, va_list ap)
 	}
 #endif /*VIPS_DEBUG*/
 
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	g_assert(vips_error_freeze_count >= 0);
 	if (!vips_error_freeze_count) {
 		if (domain)
@@ -250,7 +193,7 @@ vips_verror(const char *domain, const char *fmt, va_list ap)
 		vips_buf_vappendf(&vips_error_buf, fmt, ap);
 		vips_buf_appends(&vips_error_buf, "\n");
 	}
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 
 	if (vips__fatal)
 		vips_error_exit("vips__fatal");
@@ -259,12 +202,13 @@ vips_verror(const char *domain, const char *fmt, va_list ap)
 /**
  * vips_error:
  * @domain: the source of the error
- * @fmt: printf()-style format string for the error
+ * @fmt: `printf()`-style format string for the error
  * @...: arguments to the format string
  *
- * Format the string in the style of printf() and append to the error buffer.
+ * Format the string in the style of [`printf()`](man:printf(3)) and append to the error buffer.
  *
- * See also: vips_error_system(), vips_verror().
+ * ::: seealso
+ *     [func@error_system], [func@verror].
  */
 void
 vips_error(const char *domain, const char *fmt, ...)
@@ -280,59 +224,36 @@ vips_error(const char *domain, const char *fmt, ...)
  * vips_verror_system:
  * @err: the system error code
  * @domain: the source of the error
- * @fmt: printf()-style format string for the error
+ * @fmt: `printf()`-style format string for the error
  * @ap: arguments to the format string
  *
- * Format the string in the style of printf() and append to the error buffer.
+ * Format the string in the style of [`printf()`](man:printf(3)) and append to the error buffer.
  * Then create and append a localised message based on the system error code,
  * usually the value of errno.
  *
- * See also: vips_error_system().
+ * ::: seealso
+ *     [func@error_system].
  */
 void
 vips_verror_system(int err, const char *domain, const char *fmt, va_list ap)
 {
 	vips_verror(domain, fmt, ap);
-
-#ifdef G_OS_WIN32
-	{
-		char *buf;
-
-		if (FormatMessageA(
-				FORMAT_MESSAGE_ALLOCATE_BUFFER |
-					FORMAT_MESSAGE_IGNORE_INSERTS |
-					FORMAT_MESSAGE_FROM_SYSTEM,
-				NULL,
-				err,
-				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-				(LPSTR) &buf, 0, NULL)) {
-			vips_error(_("windows error"), "%s", buf);
-			LocalFree(buf);
-		}
-	}
-#else  /*!G_OS_WIN32*/
-	{
-		char *buf;
-
-		buf = g_locale_to_utf8(strerror(err), -1, NULL, NULL, NULL);
-		vips_error(_("unix error"), "%s", buf);
-		g_free(buf);
-	}
-#endif /*G_OS_WIN32*/
+	vips_error(_("system error"), "%s", g_strerror(err));
 }
 
 /**
  * vips_error_system:
  * @err: the system error code
  * @domain: the source of the error
- * @fmt: printf()-style format string for the error
+ * @fmt: `printf()`-style format string for the error
  * @...: arguments to the format string
  *
- * Format the string in the style of printf() and append to the error buffer.
+ * Format the string in the style of [`printf()`](man:printf(3)) and append to the error buffer.
  * Then create and append a localised message based on the system error code,
  * usually the value of errno.
  *
- * See also: vips_verror_system().
+ * ::: seealso
+ *     [func@verror_system].
  */
 void
 vips_error_system(int err, const char *domain, const char *fmt, ...)
@@ -351,9 +272,10 @@ vips_error_system(int err, const char *domain, const char *fmt, ...)
  * This function sets the glib error pointer from the vips error buffer and
  * clears it. It's handy for returning errors to glib functions from vips.
  *
- * See vips_g_error() for the inverse operation.
+ * See [func@g_error] for the inverse operation.
  *
- * See also: g_set_error(), vips_g_error().
+ * ::: seealso
+ *     [func@GLib.set_error], [func@g_error].
  */
 void
 vips_error_g(GError **error)
@@ -365,9 +287,9 @@ vips_error_g(GError **error)
 
 	/* glib does not expect a trailing '\n' and vips always has one.
 	 */
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	vips_buf_removec(&vips_error_buf, '\n');
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 
 	g_set_error(error, vips_domain, -1, "%s", vips_error_buffer());
 	vips_error_clear();
@@ -377,10 +299,11 @@ vips_error_g(GError **error)
  * vips_g_error:
  * @error: glib error pointer
  *
- * This function adds the %GError to the vips error buffer and clears it. It's
- * the opposite of vips_error_g().
+ * This function adds the [struct@GLib.Error] to the vips error buffer and clears it. It's
+ * the opposite of [func@error_g].
  *
- * See also: vips_error_g().
+ * ::: seealso
+ *     [func@error_g].
  */
 void
 vips_g_error(GError **error)
@@ -399,29 +322,31 @@ vips_g_error(GError **error)
  * Clear and reset the error buffer. This is typically called after presenting
  * an error to the user.
  *
- * See also: vips_error_buffer().
+ * ::: seealso
+ *     [func@error_buffer].
  */
 void
 vips_error_clear(void)
 {
-	g_mutex_lock(vips__global_lock);
+	g_mutex_lock(&vips__global_lock);
 	vips_buf_rewind(&vips_error_buf);
-	g_mutex_unlock(vips__global_lock);
+	g_mutex_unlock(&vips__global_lock);
 }
 
 /**
  * vips_error_exit:
- * @fmt: printf()-style format string for the message
+ * @fmt: `printf()`-style format string for the message
  * @...: arguments to the format string
  *
  * Sends a formatted error message to stderr, then sends the contents of the
  * error buffer, if any, then shuts down vips and terminates the program with
  * an error code.
  *
- * @fmt may be %NULL, in which case only the error buffer is printed before
+ * @fmt may be `NULL`, in which case only the error buffer is printed before
  * exiting.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  */
 void
 vips_error_exit(const char *fmt, ...)
@@ -457,7 +382,8 @@ vips_error_exit(const char *fmt, ...)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 on OK, or -1 on error.
  */
@@ -481,7 +407,8 @@ vips_check_uncoded(const char *domain, VipsImage *im)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 on OK, or -1 on error.
  */
@@ -509,7 +436,8 @@ vips_check_coding_noneorlabq(const char *domain, VipsImage *im)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 on OK, or -1 on error.
  */
@@ -538,7 +466,8 @@ vips_check_coding_known(const char *domain, VipsImage *im)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 on OK, or -1 on error.
  */
@@ -563,7 +492,8 @@ vips_check_coding(const char *domain, VipsImage *im, VipsCoding coding)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -588,7 +518,8 @@ vips_check_mono(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -612,7 +543,8 @@ vips_check_bands(const char *domain, VipsImage *im, int bands)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -638,7 +570,8 @@ vips_check_bands_1or3(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -665,7 +598,8 @@ vips_check_bands_atleast(const char *domain, VipsImage *im, int bands)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 on OK, or -1 on error.
  */
@@ -690,11 +624,12 @@ vips_check_bands_1orn(const char *domain, VipsImage *im1, VipsImage *im2)
  * @n: number of bands, or 1
  *
  * Check that an image has 1 or @n bands. Handy for unary operations, cf.
- * vips_check_bands_1orn().
+ * [func@check_bands_1orn].
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_check_bands_1orn().
+ * ::: seealso
+ *     [func@check_bands_1orn].
  *
  * Returns: 0 on OK, or -1 on error.
  */
@@ -718,7 +653,8 @@ vips_check_bands_1orn_unary(const char *domain, VipsImage *im, int n)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -742,7 +678,8 @@ vips_check_noncomplex(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -767,7 +704,8 @@ vips_check_complex(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -794,7 +732,8 @@ vips_check_twocomponents(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -820,7 +759,8 @@ vips_check_format(const char *domain, VipsImage *im, VipsBandFormat fmt)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -844,7 +784,8 @@ vips_check_int(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -869,7 +810,8 @@ vips_check_uint(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -897,7 +839,8 @@ vips_check_8or16(const char *domain, VipsImage *im)
  * Otherwise set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -922,7 +865,8 @@ vips_check_u8or16(const char *domain, VipsImage *im)
  * Check that the image is 8 or 16-bit unsigned integer, or float.
  * Otherwise set an error message and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -948,7 +892,8 @@ vips_check_u8or16orf(const char *domain, VipsImage *im)
  * Check that the image is unsigned int or float.
  * Otherwise set an error message and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -977,7 +922,8 @@ vips_check_uintorf(const char *domain, VipsImage *im)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1002,7 +948,8 @@ vips_check_size_same(const char *domain, VipsImage *im1, VipsImage *im2)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1029,7 +976,8 @@ vips_check_oddsquare(const char *domain, VipsImage *im)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1056,7 +1004,8 @@ vips_check_bands_same(const char *domain, VipsImage *im1, VipsImage *im2)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1083,7 +1032,8 @@ vips_check_bandno(const char *domain, VipsImage *im, int bandno)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1109,7 +1059,8 @@ vips_check_format_same(const char *domain, VipsImage *im1, VipsImage *im2)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1133,7 +1084,8 @@ vips_check_coding_same(const char *domain, VipsImage *im1, VipsImage *im2)
  *
  * Check that @n == @len.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1158,7 +1110,8 @@ vips_check_vector_length(const char *domain, int n, int len)
  * the same number of elements as there are bands in the image, or a 1-band
  * image and a many-element vector.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1195,7 +1148,8 @@ vips_check_vector(const char *domain, int n, VipsImage *im)
  * 65536 elements. Return 0 if the image will pass as a histogram, or -1 and
  * set an error message otherwise.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1229,11 +1183,12 @@ vips_check_hist(const char *domain, VipsImage *im)
  * message otherwise.
  *
  * @out is set to be @im cast to double and stored in memory. Use
- * VIPS_MATRIX() to address values in @out.
+ * [func@MATRIX] to address values in @out.
  *
  * You must unref @out when you are done with it.
  *
- * See also: VIPS_MATRIX(), vips_object_local()
+ * ::: seealso
+ *     [func@MATRIX].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1275,7 +1230,8 @@ vips_check_matrix(const char *domain, VipsImage *im, VipsImage **out)
  * Return 0 if the image will pass, or -1 and
  * set an error message otherwise.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 if OK, -1 otherwise.
  */
@@ -1301,7 +1257,8 @@ vips_check_separable(const char *domain, VipsImage *im)
  * If not, set an error message
  * and return non-zero.
  *
- * See also: vips_error().
+ * ::: seealso
+ *     [func@error].
  *
  * Returns: 0 on OK, or -1 on error.
  */
